#!/usr/bin/perl -w
# $Id: dump_evaluation.pl,v 1.5 2010/08/05 05:41:17 rnl14480 Exp $

# This program dumps the evaluation measures for all precursors in
# the database

use strict;
use lib qw(../lib/);
use BAOBAB::EvaluationWrapper;
use BAOBAB::Evaluation;
use BAOBAB::Measure;
use BAOBAB::Precursor;
use BAOBAB::Chromosome;
use BAOBAB::Organism;
use BAOBAB::OrganismWrapper;
use BAOBAB::Wrapper;
use DBI;

my $dbh = DBI->connect(
	$BAOBAB::Wrapper::dsn, $BAOBAB::Wrapper::user,
	$BAOBAB::Wrapper::password, { RaiseError => 1, AutoCommit => 0 }
) || die "Could not connect to the database : $DBI::errstr\n";
$dbh->{mysql_auto_reconnect} = 1;

die "Usage: $0 [outdir [oname]]\n" if scalar(@ARGV) > 2;

my $outdir = shift || ".";
my $foname = shift;

my %outfile;

#$dbh->trace(3);
$| = 1;

my $ow = BAOBAB::OrganismWrapper->new( dbh => $dbh );
my @oid = $ow->list();

my $mw = BAOBAB::MeasureWrapper->new( dbh => $dbh );
my @mid = $mw->list();

my $sth;
foreach my $oid (@oid) {
	my $org = $ow->read( id => $oid );
	my $oname = $org->shortname;
	next if defined $foname and $foname ne $oname;
	print "Processing $oname...\n";

	foreach my $mid (@mid) {
		my $measure = $mw->read( id => $mid );
		my $mname = $measure->name;
		print "\tProcessing $mname...\n";

		unless ( defined $outfile{$oid}{$mid} ) {
			open( $outfile{$oid}{$mid}, ">${outdir}/all_${oname}_${mname}.out" )
			  or die "Cannot open file for writing for $oname $mname\n";
		}

		my $basesql = qq{
		SELECT e.pid, e.value 
		FROM evaluation AS e, precursor AS p, chromosome AS c, organism AS o
		WHERE 
			e.mid = $mid AND 
			e.pid = p.id AND 
			p.cid = c.id AND 
			c.oid = o.id AND 
			o.id = $oid
		ORDER BY e.pid
		};

		$sth = $dbh->prepare($basesql) or die $dbh->errstr;
		$sth->execute or die $sth->errstr;
		my $count = 0;
		while ( my ( $i, $v ) = $sth->fetchrow_array() ) {
			print { $outfile{$oid}{$mid} } "$v $i\n";
			print "\r\t$count";
			$count++;
		}
		print "\n";
		$sth->finish;

		close( $outfile{$oid}{$mid} );
	}
}

$dbh->disconnect();
