#!/usr/bin/perl -w
# $Id: list_evaluation.pl,v 1.26 2010/07/07 14:03:32 rnl14480 Exp $

# This program generates lists for the values of every measures wrt
# to known/candidate precursors, as well as adequate samples of candidates

use strict;
use lib qw(../lib/);
use BAOBAB::EvaluationWrapper;
use BAOBAB::Evaluation;
use BAOBAB::Measure;
use BAOBAB::Precursor;
use BAOBAB::Chromosome;
use BAOBAB::Organism;
use BAOBAB::OrganismWrapper;
use BAOBAB::Wrapper;
use BAOBAB::Random qw(random);
use DBI;

my $dbh = DBI->connect(
	$BAOBAB::Wrapper::dsn, $BAOBAB::Wrapper::user,
	$BAOBAB::Wrapper::password, { RaiseError => 1, AutoCommit => 0 }
) || die "Could not connect to the database : $DBI::errstr\n";
$dbh->{mysql_auto_reconnect} = 1;

my $outdir = shift || ".";

my @org = qw(aga dmel ada);
my %org;
$org{$_} = 1 for (@org);
my %outfile;

$| = 1;

my $sample_size = 0;
my $rbase       = 5_000_000;
my $rpool       = random($rbase);

my $ow = BAOBAB::OrganismWrapper->new( dbh => $dbh );
my @oid = $ow->list();

my $mw = BAOBAB::MeasureWrapper->new( dbh => $dbh );
my @mid = $mw->list();

my $pw = BAOBAB::PrecursorWrapper->new( dbh => $dbh );

my $sth;
foreach my $oid (@oid) {
	my $org = $ow->read( id => $oid );
	my $oname = $org->shortname;
	next unless exists $org{$oname};
	print "Processing $oname...\n";

	my $npos = $pw->list_known( organism => $org );
	my $nneg = $pw->list_candidates(
		organism => $org,
		extra    => q[ AND precursor.id NOT IN ( SELECT cpid FROM overlap ) ]
	);
	my $chosen;
	$sample_size = $npos;
	print "Sample size = $sample_size, Neg pool = $nneg\n";
	my $nsamples = get_nsamples($oname);

	for ( my $i = 0 ; $i < $nsamples ; $i++ ) {
		my %taken = ();
		for ( my $j = 0 ; $j < $sample_size ; $j++ ) {
			my $res;
			my $repeat = 0;
			do {
				print "Already taken ($res), repeating...\n" if $repeat;
				$res = int( $rpool->() * $nneg / $rbase );
				print "Selected ($res)\n";
				$repeat++;
			} while ( exists $taken{$res} );
			$taken{$res} = 1;
			push @{ $chosen->[$i] }, $res;
		}
	}

	foreach my $mid (@mid) {
		my @neg     = ();
		my $measure = $mw->read( id => $mid );
		my $mname   = $measure->name;
		print "\tProcessing $mname...\n";

		unless ( defined $outfile{$oid}{$mid}{neg} ) {
			open( $outfile{$oid}{$mid}{neg},
				">$outdir/neg_${oname}_${mname}.out" )
			  or die "Cannot open file for writing for $oname $mname\n";
		}

		my $basesql = qq{
		SELECT e.pid, e.value 
		FROM evaluation AS e, precursor AS p, chromosome AS c, organism AS o
		WHERE 
			e.mid = $mid AND 
			e.pid = p.id AND 
			p.cid = c.id AND 
			c.oid = o.id AND 
			o.id = $oid
		};

		if ( exists $org{ $org->shortname } ) {

			unless ( defined $outfile{$oid}{$mid}{"pos"} ) {
				open( $outfile{$oid}{$mid}{"pos"},
					">$outdir/pos_${oname}_${mname}.out" )
				  or die "Cannot open file for writing for $oname $mname\n";
			}

			my $knownsql = $basesql . qq{ AND p.category = 'known' };
			my $negsql   = $basesql
			  . qq{ AND p.category = 'candidate' AND p.id NOT IN ( SELECT cpid FROM overlap ) ORDER BY e.pid};

			$sth = $dbh->prepare($knownsql) or die $dbh->errstr;
			$sth->execute or die $sth->errstr;
			my $count = 0;
			while ( my ( $i, $v ) = $sth->fetchrow_array() ) {
				print { $outfile{$oid}{$mid}{"pos"} } "$v $i\n";
				print "\r\t$count";
				$count++;
			}
			print "\n";
			$sth->finish;

			$count = 0;
			$sth = $dbh->prepare($negsql) or die $dbh->errstr;
			$sth->execute or die $sth->errstr;
			while ( my ( $i, $v ) = $sth->fetchrow_array() ) {
				print { $outfile{$oid}{$mid}{neg} } "$v $i\n";
				print "\r\t$count";
				print "HELP" unless defined $v and defined $i;
				push @neg, [ $v, $i ];
				$count++;
			}
			print "\n";
			$sth->finish;

			mkdir("${outdir}/samples_${oname}_${mname}")
			  unless ( -d "${outdir}/samples_${oname}_${mname}" );
			my $universe = scalar(@neg);
			for ( my $s = 0 ; $s < $nsamples ; $s++ ) {
				open( $outfile{$oid}{$mid}{sample},
">${outdir}/samples_${oname}_${mname}/sample.${s}_${oname}_${mname}.out"
				  )
				  or die
"Cannot open file for writing for sample $s for $oname $mname : $!\n";
				$count = 0;
				for ( my $i = 0 ; $i < $sample_size ; $i++ ) {
					my $pos   = $chosen->[$s][$i];
					my $slice = $neg[$pos];
					my ( $v, $i ) = @$slice;
					print { $outfile{$oid}{$mid}{sample} } "$v $i\n";
					print "\r\tsample $s : $count: $oname : $mname";
					$count++;
				}
				close( $outfile{$oid}{$mid}{sample} );

				print "\n";
				$sth->finish;
			}

		}
		else {

			$sth = $dbh->prepare($basesql) or die $dbh->errstr;
			$sth->execute or die $sth->errstr;
			my $count = 0;
			while ( my ( $i, $v ) = $sth->fetchrow_array() ) {
				print { $outfile{$oid}{$mid}{neg} } "$v $i\n";
				print "\r\t$count";
				$count++;
			}
			print "\n";
			$sth->finish;

		}

		close( $outfile{$oid}{$mid}{neg} );
		close( $outfile{$oid}{$mid}{"pos"} )
		  if defined $outfile{$oid}{$mid}{"pos"};
	}
}

sub get_nsamples {
	my $o = shift;
	if ( $o =~ /^planted/ ) {
		return 100;
	}
	else {
		return 1_000;
	}
}

$dbh->disconnect();
