#!/usr/bin/perl -w
# $Id: make_ROC_R_boostrap_script.pl,v 1.5 2010/09/11 14:12:06 rnl14480 Exp $

use strict;

die "Usage: $0 indir outdir fuse\n" unless scalar(@ARGV) >= 2;

my $indir  = shift || ".";
my $outdir = shift || ".";
my $fuse   = shift || 0;
my @measure = qw(amfe robust_context robust_mutation robust_folding);
my @org     = qw(aga dmel ada);

print qq[library(ROCR)] . "\n";

#print qq[library(plotrix)]."\n";
print qq[setwd("$outdir/")] . "\n";

foreach my $o (@org) {

	if ($fuse) {
		print <<EOL;
pdf("ROC_${o}_undersampling.pdf")
par(mfrow=c(2,2))
EOL
	}

	my $organism = get_organism( $o );
	foreach my $m (@measure) {

		my $transformation = get_transformation( $o, $m );
		my $nsample = get_nsample($o);
		print <<EOL;

pos <- read.table("$indir/pos_${o}_${m}.out")\$V1
predictions <- list()
labels <- list()
for (i in 1:$nsample) { 
	neg <- read.table(paste("samples_${o}_${m}/sample.",i-1,"_${o}_${m}.out",sep=''))\$V1
	posneg <- c(pos,neg)
	$transformation
	predictions[[i]] <- posneg
	labels[[i]] <- c(rep(1,length(pos)),rep(0,length(neg)))
	
}


EOL
		my $title = get_title( $o, $m, $fuse );
		my $tc = get_tc( $m, 'ba' );

		my $renderhead = '';
		my $rendertail = '';
		if ( !$fuse ) {
			$renderhead = qq[pdf("ROC_${o}_${m}_undersampling.pdf")];
			$rendertail = q[dev.off()];
		}

		print <<EOL;
$renderhead
pred <- prediction(predictions, labels)
perf <- performance(pred, "tpr", "fpr")
y <- list()
x <- list()
a <- list()
z <- list()
b <- list()
bx <- vector()
by <- vector()
ba <- vector()
for (i in 1:$nsample){
	y[[i]] <- perf\@y.values[[i]]
	x[[i]] <- perf\@x.values[[i]]
	a[[i]] <- perf\@alpha.values[[i]]
	z[[i]] <- y[[i]] - x[[i]]
	b[[i]] <- which(z[[i]] == max(z[[i]]))
	bx[i] <- x[[i]][b[[i]]][1]
	by[i] <- y[[i]][b[[i]]][1]
	ba[i] <- a[[i]][b[[i]]][1]
}
warnings()
bx <- mean(bx)
by <- mean(by)
ba <- mean(ba)
auc <- performance(pred, "auc")
aucv <- vector()
for (i in 1:$nsample){
	aucv[i] <- auc\@y.values[[i]]
}
aucv <- round(mean(aucv),2)
plot(perf, colorize=FALSE,col='blue',cez.lab=.7)
title(main="$title",cex.main=.8)
points(bx,by,col='red',pch=20)
lines(x=c(bx,bx),y=c(-.1,by),lty=2,col='red')
lines(x=c(-.1,bx),y=c(by,by),lty=2,col='red')
text(bx,0,pos=4,labels=round(bx,2),col='red')
text(0.03,by,pos=3,labels=round(by,2),col='red')
$tc
text(bx+0.25,by/2,pos=3,labels=paste("cutoff =",round(ba,2),sep=" "),col="red",adj=c(0,0))
text(bx+0.25,by/2,pos=1,labels=paste("AUC =",aucv,sep=" "),col='red',adj=c(0,0))
$rendertail
EOL
	}

	if ($fuse) {
		print <<EOL;
	mtext('${organism}',outer=T,line=-1)
	dev.off()
EOL
	}
}

sub get_transformation {
	my $o = shift;
	my $m = shift;
	my $t;
	if ( $m eq 'amfe' or $m eq 'mfei' ) {
		$t = qq[posneg <- - posneg];
	}
	elsif ( $m eq 'robust_context' ) {
		$t = '';
	}
	elsif ( $m eq 'robust_folding' ) {
		$t = qq[posneg <- 1 - (posneg / 100)];
	}
	elsif ( $m eq 'robust_mutation' ) {
		$t = qq[posneg <- 1 - (posneg / 100)];
	}

	return $t;
}

sub get_nsample {
	my $o = shift;
	if ( $o =~ /planted/ ) {
		return 100;
	}
	else {
		return 1000;
	}
}

sub get_title {
	my $o = shift;
	my $m = shift;
	my $f = shift;
	my $t;

	if ( $m eq 'amfe' ) {
		$t = q[Adjusted MFE];
	}
	elsif ( $m eq 'robust_context' ) {
		$t = q[Robustness to context];
	}
	elsif ( $m eq 'robust_folding' ) {
		$t = q[Robustness of folding];
	}
	elsif ( $m eq 'robust_mutation' ) {
		$t = q[Robustness to mutations];
	}
	elsif ( $m eq 'mfei' ) {
		$t = q[MFEI];
	}

	return $t if $f == 1;

	if ( $o eq 'aga' ) {
		$t .= q[ for A. gambiae];
	}
	elsif ( $o eq 'dmel' ) {
		$t .= q[ for D. melanogaster];
	}

	return $t;
}

sub get_organism {
	my $o = shift;

	if ( $o eq 'aga' ) {
                return  q[A. gambiae];
        }
        elsif ( $o eq 'dmel' ) {
                return q[D. melanogaster];
        }
}


sub get_tc {
	my $m = shift;
	my $v = shift;
	my $t;
	if ( $m eq 'amfe' or $m eq 'mfei' ) {
		$t = qq[$v <- - $v];
	}
	elsif ( $m eq 'robust_context' ) {
		$t = '';
	}
	elsif ( $m eq 'robust_folding' or $m eq 'robust_mutation' ) {
		$t = qq[$v <- - ($v - 1) * 100];
	}

	return $t;
}

