#!/usr/bin/perl -w
# $Id: make_composite_roc_bootstrap_script.pl,v 1.6 2010/09/11 14:12:06 rnl14480 Exp $
use strict;

die "Usage: $0 indir outdir\n" unless scalar(@ARGV) == 2;

my $indir  = shift || ".";
my $outdir = shift || ".";
my $fused  = shift || 0;

my @measure = qw(amfe robust_context robust_mutation robust_folding);
my @org     = qw(aga dmel ada);

print qq[library(ROCR)] . "\n";
print qq[setwd("$outdir/")] . "\n";

my ($header, $footer) = ('','');
my ($master_header, $master_footer, $master_titles) = ('','','');

if ($fused){
	$master_header = <<EOL;
        pdf("ROC_score_compare.pdf")
	par(mfcol=c(1,3),mar=c(5,4,4,5)+0.1)
EOL
        $master_footer = q[dev.off()]."\n";
        $header = $footer = '';
	$master_titles = '';
}

print $master_header;

foreach my $o (@org) {
	my $title   = get_title($o);
	my @m       = ();
	my $nsample = get_nsample($o);
	foreach my $m (@measure) {
		my $transformation = get_transformation( $o, $m );


# Load information about reference random genomes
#			 positive set for each measure
#			 negative samples for each measure
		print <<EOL;

random_${m} <- read.table("$indir/all_random_${o}_${m}.out")
pos_${m} <- read.table("$indir/pos_${o}_${m}.out")\$V1
id <- random_${m}\$V2
random_${m} <- random_${m}\$V1
F_${m} <- ecdf(random_${m})
pos_${m} <- F_${m}(pos_${m})
 
neg_${m} <- list()
for (i in 1:$nsample) { 
        neg_${m}[[i]] <- read.table(paste("samples_${o}_${m}/sample.",i-1,"_${o}_${m}.out",sep=''))\$V1
	neg_${m}[[i]] <- F_${m}(neg_${m}[[i]])
}

$transformation
EOL
		push @m, $m;
	}

# Load information about HHMMiR evaluation of pos/neg sets

	print <<EOL;
hhmm_pos_${o} <- read.table("$indir/hhmm_pos_${o}.out")\$V1
hhmm_pos_${o} <- 1 - hhmm_pos_${o}
hhmm_neg_${o} <- list()
for (i in 1:$nsample) {
	hhmm_neg_${o}[[i]] <- read.table(paste("$indir/hhmm_neg_",i-1,"_${o}.out",sep=""))\$V1
	hhmm_neg_${o}[[i]] <- 1 - hhmm_neg_${o}[[i]]
}

EOL

# Preparing all the information for generating ROC curves

	my $mstring = join( ",", @m );
	my $pstring = join( "*", map { "pos_$_" } @m );
	my $nstring = join( "*", map { "neg_${_}[[i]]" } @m );

	if (!$fused){
		$master_header = $master_footer = '';
		$footer = q[dev.off()]."\n";
		$header = qq[pdf("$indir/ROC_score_${o}_undersampling.pdf")];
		$master_titles = '';
	}


	print <<EOL;
		nscore <- list()
		predictions <- list()
		hpredictions <- list()
		labels <- list()
		hlabels <- list()
		pscore <- $pstring
		tsvm_data <- read.table("tsvm_${o}.data",sep=":")
		tfpr <- tsvm_data\$V1
		ttpr <- tsvm_data\$V2
		hhmm_data <- read.table("hmm_${o}.data",sep=":")
		hfpr <- hhmm_data\$V1
		htpr <- hhmm_data\$V2
		for (i in 1:$nsample){	
			nscore[[i]] <- $nstring
			posneg <- c(pscore,nscore[[i]])
			predictions[[i]] <- posneg
			hpredictions[[i]] <- c(hhmm_pos_${o},hhmm_neg_${o}[[i]])
			labels[[i]] <- c(rep(1,length(pscore)),rep(0,length(nscore[[i]])))
			hlabels[[i]] <- c(rep(1,length(hhmm_pos_${o})),rep(0,length(hhmm_neg_${o}[[i]])))
		}
		
		pred <- prediction(predictions, labels)
		hpred <- prediction(hpredictions,hlabels)
		perf <- performance(pred, "tpr", "fpr")
		hperf <- performance(hpred,"tpr","fpr")
		y <- list()
		x <- list()
		a <- list()
		z <- list()
		b <- list()
		bx <- vector()
		by <- vector()
		ba <- vector()
		for (i in 1:$nsample){
        		y[[i]] <- perf\@y.values[[i]]
        		x[[i]] <- perf\@x.values[[i]]
        		a[[i]] <- perf\@alpha.values[[i]]
        		z[[i]] <- y[[i]] - x[[i]]
        		b[[i]] <- which(z[[i]] == max(z[[i]]))
        		bx[i] <- x[[i]][b[[i]]][1]
        		by[i] <- y[[i]][b[[i]]][1]
        		ba[i] <- a[[i]][b[[i]]][1]
		}

		bx <- mean(bx)
		by <- mean(by)
		ba <- mean(ba)
		auc <- performance(pred, "auc")
		hauc <- performance(hpred,"auc")
		aucv <- vector()
		haucv <- vector()
		for (i in 1:$nsample){
        		aucv[i] <- auc\@y.values[[i]]
			haucv[i] <- hauc\@y.values[[i]]
		}
		aucv <- round(mean(aucv),2)
		haucv <- round(mean(haucv),2)

		$header
		plot(perf, colorize=FALSE, main="$title",col='blue',lty=3)
		text(bx+0.3,by/2,pos=3,labels=paste("AUC =",aucv,sep=" "),col='blue',adj=c(0,0))
		points(tfpr,ttpr,col='green',cex=.6)
		afpr <- mean(tfpr)
		atpr <- mean(ttpr)
		points(afpr,atpr,col='black',bg='chartreuse2',pch=23,cex=1.5)
		afpr <- mean(hfpr)
		atpr <- mean(htpr)
		legend('right',c("CScore","HHMMiR","TripletSVM"),fill=c('blue','magenta','chartreuse2'),col=c('blue','magenta','chartreuse2'))
		plot(hperf,col='magenta',lty=3,add=TRUE)
		plot(perf,avg="threshold",lwd=3,col='blue',add=TRUE)
		plot(hperf,avg="threshold",lwd=3,col='magenta',add=TRUE)
                text(bx+0.3,by/2,pos=1,labels=paste("AUC =",haucv,sep=" "),col='magenta',adj=c(0,0))
		$footer
EOL
}

print $master_titles;
print $master_footer;

sub get_transformation {
	my $o       = shift;
	my $m       = shift;
	my $nsample = get_nsample($o);
	my $t;

	my $invert = qq[
		pos_${m} <- 1 - pos_${m}
		for (i in 1:$nsample){
			neg_${m}[[i]] <- 1 - neg_${m}[[i]]
		}
	];

	if ( $m eq 'amfe' or $m eq 'mfei' ) {
		$t = $invert;
	}
	elsif ( $m eq 'robust_context' ) {
		$t = qq[];
	}
	elsif ( $m eq 'robust_folding' ) {
		$t = $invert;
	}
	elsif ( $m eq 'robust_mutation' ) {
		$t = $invert;
	}

	return $t;
}

sub get_title {
	my $o = shift;
	my $t;

	if ( $o eq 'aga' ) {
		$t .= q[A. gambiae];
	}
	elsif ( $o eq 'planted_metazoa' ) {
		$t .= q[metazoa];
	}
	elsif ( $o eq 'dmel' ) {
		$t .= q[D. melanogaster];
	}
	elsif ( $o eq 'ada' ) {
		$t .= q[A. darlingi];
	}
	else {
		$t = $o;
	}

	return $t;
}

sub get_nsample {
	my $o = shift;
	return 1000;
}

